<?php

namespace Ana\Boxes\Policies;

use Litepie\User\Contracts\UserPolicy;
use Ana\Boxes\Models\Box;

class BoxPolicy
{

    /**
     * Determine if the given user can view the box.
     *
     * @param UserPolicy $user
     * @param Box $box
     *
     * @return bool
     */
    public function view(UserPolicy $user, Box $box)
    {
        if ($user->canDo('boxes.box.view') && $user->isAdmin()) {
            return true;
        }

        return $box->user_id == user_id() && $box->user_type == user_type();
    }

    /**
     * Determine if the given user can create a box.
     *
     * @param UserPolicy $user
     * @param Box $box
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('boxes.box.create');
    }

    /**
     * Determine if the given user can update the given box.
     *
     * @param UserPolicy $user
     * @param Box $box
     *
     * @return bool
     */
    public function update(UserPolicy $user, Box $box)
    {
        if ($user->canDo('boxes.box.edit') && $user->isAdmin()) {
            return true;
        }

        return $box->user_id == user_id() && $box->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given box.
     *
     * @param UserPolicy $user
     * @param Box $box
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Box $box)
    {
        return $box->user_id == user_id() && $box->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given box.
     *
     * @param UserPolicy $user
     * @param Box $box
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Box $box)
    {
        if ($user->canDo('boxes.box.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given box.
     *
     * @param UserPolicy $user
     * @param Box $box
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Box $box)
    {
        if ($user->canDo('boxes.box.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
